function inc=HomogeneousSolution(nu,sigrat21,sigrat31,typ)
% MATLAB (from R2016b) function to calculate the strength (sigma_c) 
%  as a function  of the length scale (l_c) based on the homogeneous 
%  solution of the phase-field problem with spectrral energy decomposition.
%
% Input:
% nu - Poisson's ratio
% sigrat21 - principal stress ratio (sigma_2/sigma_1), where sigma_1 is the
%           largest axial tensile stress
% sigrat31 - principal stress ratio (sigma_3/sigma_1)
%
% typ - 'AT1' or 'AT2' depending on the phase-field formulation ('AT1' is
%       the linear, 'AT2' is the quadratic)
%
% Output:
% inc - is the multiplicator value: sigma_c = inc * sqrt(E*g_c/l_c), 
%       where E is Young's modulus g_c is the critical energy release
%       rate, l_c is the length scale parameter. The equation provides
%       sigma_c, the largest tensile stress (sigma_1), the material can
%       withstand
%
% Example:
% To calculate the maximum tensile strss in uniaxial tension for a
%  material with a Poisson's ratio of 0.3 and with linear damage function:
%   inc=HomogeneousSolution(0.3,0,0,'AT1')
%   sigma_c = inc * sqrt(E*g_c/l_c)
%
% ======================================================================
%
% Copyright© Gergely Molnar, CNRS, INSA of Lyon, LaMCoS, UMR5259
%
% gergely.molnar@insa-lyon.fr
% http://www.molnar-research.com
%
% This software is a computer program whose purpose is to calculate
% maximum stress as a function of the phase-field parameters and
% the material's stiffness
%
% This software is governed by the CeCILL-B license under French law and
% abiding by the rules of distribution of free software. You can  use, 
% modify and/or redistribute the software under the terms of the CeCILL-B
% license as circulated by CEA, CNRS and INRIA at the following URL
% "http://www.cecill.info". 
%
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economi% rights,  and the successive licensors  have only  limited
% liability. 
%
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specifi% status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or 
% data to be ensured and,  more generally, to use and operate it in the 
% same conditions as regards security. 
%
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL-B
%
% ======================================================================

% Checking input parametrs
if nu>=0.5 || nu<=-1.0
    error('Posson''s ratio has to be physically admissible.')
end

if sigrat21>1
    error('Principal stress ratio 2 to 1 needs to be lower than 1.')
end
if sigrat31>1
    error('Principal stress ratio 3 to 1 needs to be lower than 1.')
end

E=1;    % Normalized material constats
lc=1;
gc=1;
gam=nu/(1-2*nu);
glam=E/(2*(1+nu));
lam=E*nu/((1+nu)*(1-2*nu));

C.glam=glam;
C.lam=lam;
C.gc=gc;
C.lc=lc;
C.nu=nu;

deps1=1e-4;
toler=1e-6;

cnt1=1;
 
eps1=0;

cnt1=2;

dsig=1000;

eps1v(1)=0;
eps2v(1)=0;
eps3v(1)=0;
sig1v(1)=0;
sig2v(1)=0;
sig3v(1)=0;

while dsig>0 && cnt1<1e6
    eps1=eps1+deps1;
    deps23=1000;
    eps2=eps1*sigrat21;
    eps3=eps1*sigrat31;
    [d,ai,a]=calcd(eps1,eps2,eps3,C,typ);
    
    cnt2=1;
    while deps23>toler && cnt2<10000
        [eps2,eps3]=eps_pred(eps1,sigrat21,sigrat31,C,d,ai,a);
        [d,ai,a]=calcd(eps1,eps2,eps3,C,typ);
        [eps2n,eps3n]=eps_pred(eps1,sigrat21,sigrat31,C,d,ai,a);
        deps23=max([abs(eps2-eps2n),abs(eps3-eps3n)]);
        cnt2=cnt2+1;
    end
    
    [d,ai,a]=calcd(eps1,eps2,eps3,C,typ);
    eps1v(cnt1)=eps1;
    eps2v(cnt1)=eps2;
    eps3v(cnt1)=eps3;

    L=[2*(1-ai(1)*d)^2*glam+lam*(1-a*d)^2 lam*(1-a*d)^2 lam*(1-a*d)^2
        lam*(1-a*d)^2 2*(1-ai(2)*d)^2*glam+lam*(1-a*d)^2 lam*(1-a*d)^2
        lam*(1-a*d)^2 lam*(1-a*d)^2 2*(1-ai(3)*d)^2*glam+lam*(1-a*d)^2];
    sig=L*[eps1;eps2;eps3];

    dv(cnt1)=d;
    sig1v(cnt1)=sig(1);
    sig2v(cnt1)=sig(2);
    sig3v(cnt1)=sig(3);
    
    dsig=sig1v(cnt1)-sig1v(cnt1-1);
    cnt1=cnt1+1;
end

inc=max(sig1v)/sqrt(lc/E/gc);

end
%% Function to calculate principal strains for a given stress state
function [eps2,eps3]=eps_pred(eps1,sigrat21,sigrat31,C,d,ai,a)
glam=C.glam;
lam=C.lam;
nu=C.nu;

q=lam*(1-d*a)^2;
for i=1:3
    qi(i)=2*glam*(1-d*ai(i))^2;
end

b(1,1)=eps1*(sigrat21*q-q+qi(1)*sigrat21);
b(2,1)=eps1*(sigrat31*q-q+qi(1)*sigrat31);

A(1,1)=q-sigrat21*q+qi(2);
A(1,2)=q-sigrat21*q;
A(2,1)=q-sigrat31*q;
A(2,2)=q-sigrat31*q+qi(3);

eps=A\b;

eps2=eps(1);
eps3=eps(2);


end
%% Function to calculate damage as a function of the deformation energy
function [d,ai,a]=calcd(eps1,eps2,eps3,C,typ)
    epsv=[eps1;eps2;eps3];
    epsvV=sum(epsv);

    glam=C.glam;
    lam=C.lam;
    gc=C.gc;
    lc=C.lc;

    ai=epsv>0;
    a=epsvV>0;

    H=sum(glam*ai.*epsv.^2)+lam/2*a.*epsvV.^2;
    if contains(typ,'AT2')
        d=2*H./(gc/lc+2*H);
    elseif contains(typ,'AT1')
        d=max((2*H-gc/lc*3/8)./(2*H),0);
    end
end



