function Abaqus2Phasefield(inputPath)

% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%  When this script is used, it is the user's responsibility to check and
%                         interpret all results.
% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% 
% Abaqus2Phasefield adds the UEL option to an ABAQUS created input file
%             If the input file is created in the same order as discussed
%             in the tutorials (http://www.molnar-research.com/tutorials.html)
%             present MATLAB script finds the element definition and
%             multiplies it two times for the two UEL and the UMAT layer.
%
%    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%     The script is not able to convert all types of input files, only the
%     ones created the right order discussed in these tutorials (e.g. the
%     mesh needs to be defined on a single part).
%    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%               
%   Current version converts 2D triangular and square plane strain elements
%           and 3D tetrahedral and brick solid elements
%
%  -----  INPUT variables are the following -------------------------------
% inputPath - is a string which point to the input file created by ABAQUS,
%				if left blank [], the script opens an inquiry
%
% -------------------------------------------------------------------------
%
% BASIC structure of the script:
% 1)  Find and store phse-field material parameters.
% 2)  Identify Dirichlet boundary conditions in the phase-field
% 3)  Identify, multiply and reorder elements with phase-field definitions.
%           (the code support built-in material models as well;
%            all sets with phase-field fracture should be identified with
%            the prefix "uel_")
% 4)  Change original set defnitions according to the new element order
% 5)  Write connectivity table to external file (for Lagrange multipliers)
% 6)  Assign material properties
% 7)  Continue copying the original input file until the end of the assembly
%           part. Add an additional element set summing up all the UMAT
%           elements.
% 8)  Modify time integration controll.
% 9)  Finishes the with the simulation steps
% 10) Make a copy of the FORTAN file and change the number of UEL elements
%     (N_ELEM) in it.
% -------------------------------------------------------------------------
%
% ======================================================================
%
% Copyright© Gergely Molnar, CNRS, INSA of Lyon, LaMCoS, UMR5259
%
% gergely.molnar@insa-lyon.fr
% http://www.molnar-research.com
%
% This software is a computer program whose purpose is to calculate
% maximum stress as a function of the phase-field parameters and
% the material's stiffness
%
% This software is governed by the CeCILL-B license under French law and
% abiding by the rules of distribution of free software. You can  use, 
% modify and/or redistribute the software under the terms of the CeCILL-B
% license as circulated by CEA, CNRS and INRIA at the following URL
% "http://www.cecill.info". 
%
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability. 
%
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or 
% data to be ensured and,  more generally, to use and operate it in the 
% same conditions as regards security. 
%
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL-B
%
% ======================================================================
%

narginLoc=nargin;
% ---------------- Asking for the input file -------------------
if narginLoc==0
    [FileName,PathName,FilterIndex] = uigetfile('*.inp');
    inputPath=[PathName FileName];
end
if isa(inputPath,'char')~=1
    error('Provide a valid input file.')
end
if isempty(inputPath)==1
    [FileName,PathName,FilterIndex] = uigetfile('*.inp');
    inputPath=[PathName FileName];
end

% Find all phase-field material properties
inputName=inputPath;
fid=fopen(inputName,'rt');
a=fgets(fid);

while(ischar(a))
    if strfind(a,'** MATERIALS')~=0
        break
    end
    a=fgets(fid);
end
dens=[];
MatProp=[];
cnt=1;
cntPH=1;
PHbon={};
while(ischar(a))
    if strfind(a,'*Material')~=0
        ini=strfind(a,'name=')+5;
        MatName{cnt}=a(ini:end);
        a=fgets(fid);
        while(ischar(a))
            if strfind(a,'*Density')~=0
                a=fgets(fid);
                dens(cnt)=str2num(a);
            elseif strfind(a,'*User Material')~=0
                a=fgets(fid);
                MatProp(cnt,:)=str2num(a);
                break
            end
            a=fgets(fid);
        end
        cnt=cnt+1;
    elseif strfind(a,', 6, 6, 1.')~=0
        % Finding all the sets where the phase-field is predefined on the boundary
        bin=strfind(a,',');
        PHbon{cntPH}=a(1:bin(1)-1);
        cntPH=cntPH+1;
    end
    a=fgets(fid);
end
fclose('all');

lc=MatProp(:,3);
gc=MatProp(:,4);
atswt=MatProp(:,5);


% Finding nodes included in the node sets for the phase-field boundaries
NodeListPH=[];
if isempty(PHbon)~=1
    fid=fopen(inputName,'rt');
    a=fgets(fid);
    while(ischar(a))
        if strfind(a,'** ASSEMBLY')~=0
            break
        end
        a=fgets(fid);
    end
    while(ischar(a))
        if strfind(a,'*Nset')~=0
            bin=strfind(a,' nset=');
            bin2=strfind(a,',');
            if length(bin2)>1
                NameLoc=a(bin+6:bin2(2)-1);
            else
                NameLoc=a(bin+6:end);
            end
            index = find(strcmp(PHbon, NameLoc));
            if isempty(index)==0
                if strfind(a,'generate')~=0
                    a=fgets(fid);
                    b=str2num(a);
                    NodeListPH=[NodeListPH, b(1):b(3):b(2)];
                    a=fgets(fid);
                else
                    a=fgets(fid);
                    while isempty(strfind(a,'*'))
                        b=str2num(a);
                        NodeListPH=[NodeListPH, b];
                        a=fgets(fid);
                    end
                end
            else
            a=fgets(fid);    
            end
        else
            a=fgets(fid);
        end
    end
    NodeListPH=unique(NodeListPH);
    fclose('all');
end


% ---------------- Initializing output file -------------------
inputName=inputPath;
output=[inputPath(1:end-4) '_UEL.inp'];

fid=fopen(inputName,'rt');
fout1 = fopen(output, 'wt');
a=fgets(fid);

cnt=0;

% ---------------- Start reading input file -------------------
cntE=1;
while(ischar(a))

% ------ Repeating nodes ------
while(ischar(a))
    if strfind(a,'*Element')~=0
        break
    end
    fprintf(fout1,a);    
    a=fgets(fid);
end

% ------ Defintion of UEL types ------
if contains(a,'CPE3')
   % triangular elements 
    fprintf(fout1,['*********************** TRIANGULAR ****************************\n']);
    fprintf(fout1,['*User element, nodes=3, type=U1, properties=3, coordinates=2, VARIABLES=5\n']);
    fprintf(fout1,['6,7\n']);
    NnodeE(cntE)=1;
elseif contains(a,'CPE4')
   % rectangular elements 
    fprintf(fout1,['*********************** RECTENGULAR ***************************\n']);
    fprintf(fout1,['*User element, nodes=4, type=U2, properties=3, coordinates=2, VARIABLES=12\n']);
    fprintf(fout1,['6,7\n']);
    NnodeE(cntE)=2;
elseif contains(a,'C3D4')
   % rectangular elements 
    fprintf(fout1,['*********************** TETRAHEDRAL ***************************\n']);
    fprintf(fout1,['*User element, nodes=4, type=U3, properties=3, coordinates=3, VARIABLES=6\n']);
    fprintf(fout1,['6,7\n']);
    NnodeE(cntE)=3;    
elseif contains(a,'C3D8')
   % rectangular elements 
    fprintf(fout1,['*********************** BRICK ***************************\n']);
    fprintf(fout1,['*User element, nodes=8, type=U4, properties=3, coordinates=3, VARIABLES=16\n']);
    fprintf(fout1,['6,7\n']);
    NnodeE(cntE)=4;      
end

NElemType{1}='CPE3';
NElemType{2}='CPE4';
NElemType{3}='C3D4';
NElemType{4}='C3D8';

a=fgets(fid);

% ------ Uploading original mesh ------
    cnt=0;
    while isempty(strfind(a,'*Nset,')) && isempty(strfind(a,'*Element,'))
        cnt=cnt+1;
        ElMat{cntE}(cnt,:)=str2num(a);
        a=fgets(fid);
    end
    nElem(cntE)=length(ElMat{cntE}(:,1));

    fprintf(fout1,['***************************************************************\n']);

    if isempty(strfind(a,'*Element,'))==0
        cntE=cntE+1;
    else
        break
    end
end


TypeElem=zeros(sum(nElem),1);
for i=1:cntE
    TypeElem(ElMat{i}(:,1))=NnodeE(i);
end

% ------ All elements ------
nElemAll=sum(nElem);

% Uploading Elsets for different materials
cnt2=1;
while isempty(strfind(a,'** Section'))
   while isempty(strfind(a,'*Elset,'))
       a=fgets(fid);
   end
   bin=strfind(a,'elset=');
   Ename=a(bin+6:end-1);
   if isempty(strfind(Ename,','))==0
      bin=strfind(Ename,',');
      Ename=Ename(1:bin-1);
   end
   ElstName{cnt2}=Ename;
%   fprintf(fout1,a);
   ElSeT{cnt2}{1}=a;
   a=fgets(fid);
   cnt3=2;
   while isempty(strfind(a,'** Section:')) && isempty(strfind(a,'*Nset,'))
       ElSeT{cnt2}{cnt3}=a;
%       fprintf(fout1,a);
       a=fgets(fid);
       cnt3=cnt3+1;
   end
   Nline(cnt2)=cnt3-1;
   cnt2=cnt2+1;
end
%fprintf(fout1,['***************************************************************\n']);

nMat=length(ElstName);

% Converting uploaded strings to actual numbers
for i=1:nMat
    clear noLoc
   if isempty(strfind(ElSeT{i}{1},'generate'))
       noLoc=[];
       for j=2:length(ElSeT{i})
           noLoc=[noLoc,str2num(ElSeT{i}{j})];
       end
       genopt(i)=0;
   else
       noLoc=str2num(ElSeT{i}{2});
       genopt(i)=1;       
   end
   noElem{i}=noLoc;
   if isempty(strfind(ElSeT{i}{1},'uel_'))
      isuel(i)=0;
   else
      isuel(i)=1;
   end
end

if nnz(isuel)<1
    error('No UEL sets were found.')
end



% Choosing element sets for UELs and reordering them accordingly
% we need to have the UELs first, then the rest of the model
allUEL=[];
allnonUEL=[];
for i=1:nMat
    if isuel(i)==1
        if genopt(i)==0
            allUEL=[allUEL, noElem{i}];
        else
            allUEL=[allUEL, noElem{i}(1):noElem{i}(3):noElem{i}(2)];
        end
    else
        if genopt(i)==0
            allnonUEL=[allnonUEL, noElem{i}];
        else
            allnonUEL=[allnonUEL, noElem{i}(1):noElem{i}(3):noElem{i}(2)];
        end        
    end
end

% Phase-field elements
% 2D elements
OrderElemUELT=intersect(find(TypeElem==1),allUEL);
TriUEL=length(OrderElemUELT);
OrderElemUELQ=intersect(find(TypeElem==2),allUEL);
QuadUEL=length(OrderElemUELQ);

% 3D elements
OrderElemUELTet=intersect(find(TypeElem==3),allUEL);
TetUEL=length(OrderElemUELTet);
OrderElemUELB=intersect(find(TypeElem==4),allUEL);
BrickUEL=length(OrderElemUELB);

% Number of all phase-field elements
noUEL=TriUEL+QuadUEL+TetUEL+BrickUEL;


% Non phase-field elements
% 2D elements
OrderElemnonUELT=intersect(find(TypeElem==1),allnonUEL);
TrinonUEL=length(OrderElemnonUELT);
OrderElemnonUELQ=intersect(find(TypeElem==2),allnonUEL);
QuadnonUEL=length(OrderElemnonUELQ);

% 3D elements
OrderElemnonUELTet=intersect(find(TypeElem==3),allnonUEL);
TetnonUEL=length(OrderElemnonUELTet);
OrderElemnonUELB=intersect(find(TypeElem==4),allnonUEL);
BricknonUEL=length(OrderElemnonUELB);



if TriUEL>0
    locPosType=find(NnodeE==1);
    [~,idxT]=intersect(ElMat{locPosType}(:,1),OrderElemUELT);
end
if QuadUEL>0
    locPosType=find(NnodeE==2);
    [~,idxQ]=intersect(ElMat{locPosType}(:,1),OrderElemUELQ);
end
if TetUEL>0
    locPosType=find(NnodeE==3);
    [~,idxTet]=intersect(ElMat{locPosType}(:,1),OrderElemUELTet);
end
if BrickUEL>0
    locPosType=find(NnodeE==4);
    [~,idxB]=intersect(ElMat{locPosType}(:,1),OrderElemUELB);
end


if TrinonUEL>0
    locPosType=find(NnodeE==1);
    [~,idxnT]=intersect(ElMat{locPosType}(:,1),OrderElemnonUELT);
end
if QuadnonUEL>0
    locPosType=find(NnodeE==2);
    [~,idxnQ]=intersect(ElMat{locPosType}(:,1),OrderElemnonUELQ);
end
if TetnonUEL>0
    locPosType=find(NnodeE==3);
    [~,idxnTet]=intersect(ElMat{locPosType}(:,1),OrderElemnonUELTet);
end
if BricknonUEL>0
    locPosType=find(NnodeE==4);
    [~,idxnB]=intersect(ElMat{locPosType}(:,1),OrderElemnonUELB);
end

cnt=1;

% Normal elements and new element order
NewOrder=zeros(nElemAll,1);

% -------- Creating connectivity table ------------
connec=zeros(1,8);

% Writing phase-field elements to file
if TriUEL>0
    fprintf(fout1,['***************************************************************\n']);
    fprintf(fout1,['*Element, type=U1\n']);
    locPosType=find(NnodeE==1);
    for i=1:TriUEL
        j=idxT(i);
        fprintf(fout1,[num2str(cnt) ', ' num2str(ElMat{locPosType}(j,2:end-1),'%d, ') ' ' num2str(ElMat{locPosType}(j,end)) '\n']);
        connec(cnt,1:3)=ElMat{locPosType}(j,2:end);
        uelPH(OrderElemUELT(i))=cnt;
        cnt=cnt+1;
    end
end
if QuadUEL>0
    fprintf(fout1,['***************************************************************\n']);
    fprintf(fout1,['*Element, type=U2\n']);
    locPosType=find(NnodeE==2);
    for i=1:QuadUEL
        j=idxQ(i);
        fprintf(fout1,[num2str(cnt) ', ' num2str(ElMat{locPosType}(j,2:end-1),'%d, ') ' ' num2str(ElMat{locPosType}(j,end)) '\n']);
        connec(cnt,1:4)=ElMat{locPosType}(j,2:end);
        uelPH(OrderElemUELQ(i))=cnt;
        cnt=cnt+1;
    end
end
if TetUEL>0
    fprintf(fout1,['***************************************************************\n']);
    fprintf(fout1,['*Element, type=U3\n']);
    locPosType=find(NnodeE==3);
    for i=1:TetUEL
        j=idxTet(i);
        fprintf(fout1,[num2str(cnt) ', ' num2str(ElMat{locPosType}(j,2:end-1),'%d, ') ' ' num2str(ElMat{locPosType}(j,end)) '\n']);
        connec(cnt,1:4)=ElMat{locPosType}(j,2:end);
        uelPH(OrderElemUELTet(i))=cnt;
        cnt=cnt+1;
    end
end
if BrickUEL>0
    fprintf(fout1,['***************************************************************\n']);
    fprintf(fout1,['*Element, type=U4\n']);
    locPosType=find(NnodeE==4);
    for i=1:BrickUEL
        j=idxB(i);
        fprintf(fout1,[num2str(cnt) ', ' num2str(ElMat{locPosType}(j,2:end-1),'%d, ') ' ' num2str(ElMat{locPosType}(j,end)) '\n']);
        connec(cnt,1:8)=ElMat{locPosType}(j,2:end);
        uelPH(OrderElemUELB(i))=cnt;
        cnt=cnt+1;
    end
end

% UMAT elements
if TriUEL>0
    fprintf(fout1,['********************* UMAT tri ***********************************\n']);
    fprintf(fout1,['*Element, type=CPE3\n']);
    locPosType=find(NnodeE==1);
    for i=1:TriUEL
        j=idxT(i);
        NewOrder(OrderElemUELT(i))=cnt;
        fprintf(fout1,[num2str(cnt) ', ' num2str(ElMat{locPosType}(j,2:end-1),'%d, ') ' ' num2str(ElMat{locPosType}(j,end)) '\n']);
        uelSS(OrderElemUELT(i))=cnt;
        cnt=cnt+1;
    end
end
if QuadUEL>0
    fprintf(fout1,['********************* UMAT quad ***********************************\n']);
    fprintf(fout1,['*Element, type=CPE4\n']);
    locPosType=find(NnodeE==2);
    for i=1:QuadUEL
        j=idxQ(i);
        NewOrder(OrderElemUELQ(i))=cnt;
        fprintf(fout1,[num2str(cnt) ', ' num2str(ElMat{locPosType}(j,2:end-1),'%d, ') ' ' num2str(ElMat{locPosType}(j,end)) '\n']);
        uelSS(OrderElemUELQ(i))=cnt;
        cnt=cnt+1;
    end
end
if TetUEL>0
    fprintf(fout1,['********************* UMAT tet ***********************************\n']);
    fprintf(fout1,['*Element, type=C3D4\n']);
    locPosType=find(NnodeE==3);
    for i=1:TetUEL
        j=idxTet(i);
        NewOrder(OrderElemUELTet(i))=cnt;
        fprintf(fout1,[num2str(cnt) ', ' num2str(ElMat{locPosType}(j,2:end-1),'%d, ') ' ' num2str(ElMat{locPosType}(j,end)) '\n']);
        uelSS(OrderElemUELTet(i))=cnt;
        cnt=cnt+1;
    end
end
if BrickUEL>0
    fprintf(fout1,['********************* UMAT brick ***********************************\n']);
    fprintf(fout1,['*Element, type=C3D8\n']);
    locPosType=find(NnodeE==4);
    for i=1:BrickUEL
        j=idxB(i);
        NewOrder(OrderElemUELB(i))=cnt;
        fprintf(fout1,[num2str(cnt) ', ' num2str(ElMat{locPosType}(j,2:end-1),'%d, ') ' ' num2str(ElMat{locPosType}(j,end)) '\n']);
        uelSS(OrderElemUELB(i))=cnt;
        cnt=cnt+1;
    end
end


% Non phase-field elements
if TrinonUEL>0
    fprintf(fout1,['********************* non UMAT tri ***********************************\n']);
    fprintf(fout1,['*Element, type=CPE3\n']);
    locPosType=find(NnodeE==1);
    for i=1:TrinonUEL
        j=idxnT(i);
        NewOrder(OrderElemnonUELT(i))=cnt;
        fprintf(fout1,[num2str(cnt) ', ' num2str(ElMat{locPosType}(j,2:end-1),'%d, ') ' ' num2str(ElMat{locPosType}(j,end)) '\n']);
        cnt=cnt+1;
    end
end    
if QuadnonUEL>0
fprintf(fout1,['********************* non UMAT quad ***********************************\n']);
fprintf(fout1,['*Element, type=CPE4\n']);
    locPosType=find(NnodeE==2);
    for i=1:QuadnonUEL
        j=idxnQ(i);
        NewOrder(OrderElemnonUELQ(i))=cnt;
        fprintf(fout1,[num2str(cnt) ', ' num2str(ElMat{locPosType}(j,2:end-1),'%d, ') ' ' num2str(ElMat{locPosType}(j,end)) '\n']);
        cnt=cnt+1;
    end
end    
if TetnonUEL>0
    fprintf(fout1,['********************* non UMAT tet ***********************************\n']);
    fprintf(fout1,['*Element, type=C3D4\n']);
    locPosType=find(NnodeE==3);
    for i=1:TetnonUEL
        j=idxnTet(i);
        NewOrder(OrderElemnonUELTet(i))=cnt;
        fprintf(fout1,[num2str(cnt) ', ' num2str(ElMat{locPosType}(j,2:end-1),'%d, ') ' ' num2str(ElMat{locPosType}(j,end)) '\n']);
        cnt=cnt+1;
    end
end    
if BricknonUEL>0
fprintf(fout1,['********************* non UMAT brick ***********************************\n']);
fprintf(fout1,['*Element, type=C3D8\n']);
    locPosType=find(NnodeE==4);
    for i=1:BricknonUEL
        j=idxnB(i);
        NewOrder(OrderElemnonUELB(i))=cnt;
        fprintf(fout1,[num2str(cnt) ', ' num2str(ElMat{locPosType}(j,2:end-1),'%d, ') ' ' num2str(ElMat{locPosType}(j,end)) '\n']);
        cnt=cnt+1;
    end
end    

AllElem=cnt-1;

fprintf(fout1,['***************************************************************\n']);


% Writing connectivity table to file
maxN=max(connec(:));
for i=1:maxN
    bin=find(i==connec);
    if length(bin)>1
        connec(bin(2:end))=0;
    end
end

% Substracting PH boundary nodes
for i=NodeListPH
    bin=find(i==connec);
    connec(bin)=-1;
end

dlmwrite([output(1:end-4) '_connec.dat'],connec,',')


% ------ Redefining element sets ------
for i=1:nMat
   if isuel(i)==0
       % ---------- Non-UEL sets --------
       if genopt(i)==0
          fprintf(fout1,ElSeT{i}{1});
          locvals=[];
          for j=2:length(ElSeT{i})
            locvals=[locvals str2num(ElSeT{i}{j})];
          end
          bnew=NewOrder(locvals)';
            nline=floor(length(bnew)/16);
            for j=1:nline
                fprintf(fout1,[num2str(bnew((j-1)*16+1:j*16),'%1.0i, ') '\n']);
            end
            str=num2str(bnew((nline*16)+1:end),'%1.0i, ');
            fprintf(fout1,[str(1:end-1) '\n']);
       else
          fprintf(fout1,ElSeT{i}{1});
          borig=str2num(ElSeT{i}{2});
          bnew=NewOrder(borig)';
          fprintf(fout1,[num2str(bnew(1)) ', ' num2str(bnew(2)) ', ' num2str(borig(3))  '\n']);
       end
   else
       % ---------- UEL sets --------
       if genopt(i)==0
          % Displacement elements 
          fprintf(fout1,ElSeT{i}{1});
          locvals=[];
          for j=2:length(ElSeT{i})
            locvals=[locvals str2num(ElSeT{i}{j})];
          end
          
          bnew=uelSS(locvals);
          nline=floor(length(bnew)/16);
          for j=1:nline
            fprintf(fout1,[num2str(bnew((j-1)*16+1:j*16),'%1.0i, ') '\n']);
          end
          str=num2str(bnew((nline*16)+1:end),'%1.0i, ');
          fprintf(fout1,[str(1:end-1) '\n']);
          
          % Phase field elements 
          fprintf(fout1,[ElSeT{i}{1}(1:end-1) '_PH\n']);
          bnew=uelPH(locvals);
          nline=floor(length(bnew)/16);
          for j=1:nline
            fprintf(fout1,[num2str(bnew((j-1)*16+1:j*16),'%1.0i, ') '\n']);
          end
          str=num2str(bnew((nline*16)+1:end),'%1.0i, ');
          fprintf(fout1,[str(1:end-1) '\n']);   
     else
          % Displacement elements 
          fprintf(fout1,ElSeT{i}{1});
          borig=str2num(ElSeT{i}{2});
          dborig=borig(3);
          borig=borig(1):dborig:borig(2);
          bnew=uelSS(borig);
          fprintf(fout1,[num2str(min(bnew)) ', ' num2str(max(bnew)) ', ' num2str(dborig)  '\n']);
          
          % Phase field elements
          fprintf(fout1,['*Elset, elset=' ElstName{i} '_PH, generate\n' ]);
          borig=str2num(ElSeT{i}{2});
          dborig=borig(3);
          borig=borig(1):dborig:borig(2);
          bnew=uelPH(borig);
          fprintf(fout1,[num2str(min(bnew)) ', ' num2str(max(bnew)) ', ' num2str(dborig)  '\n']);         
          
       end
   end
end

% ------ Sets and material properties ------
fprintf(fout1,['***************************************************************\n']);
fprintf(fout1,['********************** ASSIGNING MATERIAL PROP ****************\n']);


fprintf(fout1,['***************************************************************\n']);
k=1;
for kset=find(isuel==1)
    fprintf(fout1,['*Uel property, elset=' ElstName{kset} '_PH\n']);
    fprintf(fout1,[num2str(lc(k)) ', ' num2str(gc(k)) ', ' num2str(atswt(k)) '\n']);
    k=k+1;
end
fprintf(fout1,['**lc, gc, AT\n']);
fprintf(fout1,['***************************************************************\n']);

%a=fgets(fid);

while ~contains(a,'*End Part')
   fprintf(fout1,a);        
   a=fgets(fid);
end

while(ischar(a))
    if strfind(a,'*Instance, name=')~=0
        break
    end
    fprintf(fout1,a);    
    a=fgets(fid);
end

InstanceName=a(strfind(a,'*Instance, name=')+16:strfind(a,', part=')-1);


while(ischar(a))
    if strfind(a,'*Elset')~=0
        if strfind(a,'generate')~=0
            a = strrep(a,', generate','');
            fprintf(fout1,a);
            a=fgets(fid);
            b=str2num(a);
            bnew=NewOrder(b(1):b(3):b(2))';
            nline=floor(length(bnew)/16);
            for i=1:nline
                fprintf(fout1,[num2str(bnew((i-1)*16+1:i*16),'%1.0i, ') '\n']);
            end
            str=num2str(bnew((nline*16)+1:end),'%1.0i, ');
            fprintf(fout1,[str(1:end-1) '\n']);
            a=fgets(fid);
        else
            fprintf(fout1,a);
            a=fgets(fid);
            while ~contains(a,'*')
                b=str2num(a);
                b=NewOrder(b)';
                fprintf(fout1,[num2str(b,'%1.0i, ') '\n']);
                a=fgets(fid);
            end
        end
    else
        if strfind(a,'*End Assembly')~=0
            break
        end
        fprintf(fout1,a);    
        a=fgets(fid);
    end
end


while(ischar(a))
    if strfind(a,'*End Assembly')~=0
        break
    end
    fprintf(fout1,a);    
    a=fgets(fid);
end

fprintf(fout1,['**\n']);

fprintf(fout1,['*Elset, elset=umatelem, instance=' InstanceName ', generate\n']);
fprintf(fout1,[num2str(noUEL+1) ', ' num2str(noUEL*2) ', 1\n']);
fprintf(fout1,['*Elset, elset=allelem, instance=' InstanceName ', generate\n']);
fprintf(fout1,[num2str(noUEL+1) ', ' num2str(AllElem) ', 1\n']);
fprintf(fout1,['**\n*End Assembly\n']);
fprintf(fout1,['***************************************************************\n']);


a=fgets(fid);

while(ischar(a))
    while(ischar(a))
        if strfind(a,'*Output, history')~=0
            break
        end
        fprintf(fout1,a);    
        a=fgets(fid);
    end
    fprintf(fout1,a);
    a=fgets(fid);
    
    while(ischar(a))
        if strfind(a,'*End Step')~=0
            fprintf(fout1,['**\n*CONTROLS, PARAMETERS=TIME INCREMENTATION\n']);
            fprintf(fout1,['25, 100, , 100, , , , 100, , ,\n']);
            break
        end
        fprintf(fout1,a);    
        a=fgets(fid);
    end
    fprintf(fout1,a);    
    a=fgets(fid);

end
fclose('all');

% --------------- Start modifying the UEL fortran file -------------------

% determining if the OS if Lunix, Windows of Mac
comp = computer;
if strfind(comp,'WIN')~=0
    comptype=1;     % windows
elseif strfind(comp,'LNX')~=0
    comptype=2;     % linux
elseif strfind(comp,'MAC')~=0
    comptype=3;     % mac
else
    comptype=input('Please provide OS type: 1 - Microsoft Windows; 2 - Linux; 3 - Apple Mac: ');     % asking user
    if comptype==1 || comptype==2 || comptype==3
    else
        comptype=input('Wrong type.\n Please provide OS type: 1 - Microsoft Windows; 2 - Linux; 3 - Apple Mac: ');     % asking user
        if comptype==1 || comptype==2 || comptype==3
        else
        error('Wrong type. Try again.')
    end
    end
end

fullp=mfilename('fullpath');
if isempty(fullp)~=1
    input2=[fullp(1:end-18) '/PhaseFieldUEL/PH_UMAT_PE.f'];
else
    fullp=matlab.desktop.editor.getActiveFilename;
    input2=[fullp(1:end-20) '/PhaseFieldUEL/PH_UMAT_PE.f'];
end

if comptype==1
    output=[inputName(1:end-4) '_UEL_PE.for'];
else
    output=[inputName(1:end-4) '_UEL_PE.f'];
end


fid=fopen(input2,'rt');
fout1 = fopen(output, 'wt');
a=fgets(fid);

while(ischar(a))
    if strfind(a,'N_ELEM=')~=0
        strs=strfind(a,'N_ELEM=');
        ends=strs+6;
        a=[a(1:ends) num2str(noUEL) a(ends+2:end)];
    end
    fprintf(fout1,a);    
    a=fgets(fid);
end

fclose('all');

disp('Conversion finished')

return